BMapCom.Controller.create(function() {

    var pageParamCache = {},
        commentSource;

    var $ = function(q) {
        return document.querySelector(q);
    };

    var filmDelegator = {
        search: function(movie_id) {
            var self = this,
                LOCAL_STORAGE_KEY = "lsk-film-" + movie_id,
                resultData = BMapCom.storage.getLocalStorage(LOCAL_STORAGE_KEY),
                url;

            if (resultData) {
                //命中缓存
                self.onSourceLoadFinish(resultData);
            } else {
                url = "http://map.baidu.com/detail?qt=movie&act=film&movie_id=" + movie_id;

                return BMapCom.Loader.jsonp(url)
                    .ok(function(result) {
                        if (result.data.errorNo == 0) {
                            BMapCom.storage.setLocalStorage(LOCAL_STORAGE_KEY, result.data, 5);
                            self.onSourceLoadFinish(result.data);
                        }
                    })
                    .error(function() {
                        self.onSourceLoadFailed();
                    });
            }
        }
    }

    var commentDelegator = {
        search: function(opts) {
            var self = this,
                url;

            url = "http://client.map.baidu.com/movie/info/lightapp/comments/?movie_id="
                + opts.movie_id + '&page=' + pageParamCache.pageNum;

            return BMapCom.Loader.jsonp(url)
                .ok(function(result) {
                    if (result.data.errorNo == 0) {
                        self.onSourceLoadFinish(result.data, opts.type);
                    }
                })
                .error(function() {
                    self.onSourceLoadFailed();
                });
        }
    }

    var dataFixer = {
        fixFilmData: function(filmData) {
            var data = {},
                is_imax,
                is_3d,
                desc;

            if (!filmData || filmData.errorNo != 0) {
                data.movie_picture = '../img/loading_pic_error.png';
                data.movie_name = "暂无数据";
                data.movie_score = "暂无数据";
                data.movie_tags = "暂无数据";
                data.movie_release_date = "暂无数据";
                data.movie_starring = "暂无数据";
                data.movie_description = "暂无影讯";
                data.movie_director = "暂无数据";
                data.movie_length = "暂无数据";
                data.star = "暂无数据";
            }else{
                data = filmData;
                is_imax = data.is_imax ? 'IMAX' : '';
                is_3d = data.movie_type == '3D' ? '3D' : '';
                data.film_type = is_imax + (is_3d && is_imax ? ' &nbsp;|&nbsp; ' : '') + is_3d;

                desc = data.movie_description;
                if(desc.length > 120){
                    data.movie_description = desc.substring(0,120) + '......';
                    pageParamCache.movieDescAll = desc;
                    pageParamCache.movieDescShort = desc.substring(0,120);
                }
            }

            return data;
        }
    }

    var renderPage = function(filmPage, filmTpl, filmData) {
        var filmFixData = dataFixer.fixFilmData(filmData),
            showFilm;

        $('#film-info').innerHTML = filmTpl.filmInfoPanel(filmFixData);
        $('#film-intro').innerHTML = filmTpl.filmIntroPanel(filmFixData);

        if (pageParamCache.movieDescShort) {
            $('#story-arrow').style.display = 'block';
            showFilm = new BMapCom.EventDelegater($('#nostory'));
            showFilm.on('showFilm:tap', function() {
                if ($('#story-arrow').className == 'arrow-down') {
                    $('#story').innerHTML = pageParamCache.movieDescAll;
                    $('#story-arrow').className = 'arrow-up';
                }else{
                    $('#story').innerHTML = pageParamCache.movieDescShort;
                    $('#story-arrow').className = 'arrow-down';
                }
            });
        }
    }

    var renderComment = function(type, filmPage, filmTpl, data){
        var comment = $('#comBox');

        if (type == 'more') {
            $('#comment').innerHTML += filmTpl.commentItem(data);
        }else {
            comment.innerHTML = filmTpl.commentPanel();
            $('#comment').innerHTML = filmTpl.commentItem(data);
        }
    }

    var filmController = {
        initWithPage: function(filmPage, filmTpl) {
            var self = this,
                filmSource = new BMapCom.DataSource();

            filmSource
                .delegate(filmDelegator)
                .delegate({
                    onSourceLoadFinish: function(result) {
                        renderPage(filmPage, filmTpl, result);
                        BMapCom.Kernel.notify('webPageReady');
                    },
                    startSourceLoad: function(opts) {
                        this.search(opts);
                    }
                });

            commentSource = new BMapCom.DataSource();
            commentSource
                .delegate(commentDelegator)
                .delegate({
                    onSourceLoadFinish: function(result, type) {
                        renderComment(type, filmPage, filmTpl, result);
                        if (type == 'first') {
                            self.bindPageEvent();
                        }
                    },
                    startSourceLoad: function(movie_id) {
                        this.search(movie_id);
                    }
                });

            filmPage
                .delegate({
                    shouldWaiting: function() {
                        return true;
                    },
                    onCreate: function(pageParam, ready) {
                        BMapCom.Stat.sendOnline({
                            "da_src" : "moviedetailpg",
                            "da_trd" : "movie",
                            "da_model" : "plugin_movie",
                            "da_act": "show"
                        });

                        filmPage.append(filmTpl.body());
                        if (BMapCom.util.platform.isAndroid()) {
                            $('#footer-space').className = 'footer-space';
                        }
                        pageParamCache = pageParam;
                        pageParamCache.pageNum = 1;
                        filmSource.update(pageParam.movie_id);
                        commentSource.update({
                            movie_id: pageParam.movie_id,
                            type: 'first'
                        });
                    },
                    onDestory: function() {}
                });
        },
        bindPageEvent: function() {
            var evt = new BMapCom.EventDelegater($('#getMore'));

            evt.on('loadMore:tap', function() {
                pageParamCache.pageNum++;
                commentSource.update({
                    movie_id: pageParamCache.movie_id,
                    type: 'more'
                });
            });
        }
    }

    return filmController
});